@@ -7,7 +7,7 @@ RSF_HOST = 'http://rsf.qbox.me'  # 列举操作Host
 API_HOST = 'http://api.qiniu.com'  # 数据处理操作Host
 UC_HOST = 'https://uc.qbox.me'  # 获取空间信息Host
 
-_BLOCK_SIZE = 1024 * 1024 * 4  # 断点续上传分块大小，该参数为接口规格，暂不支持修改
+_BLOCK_SIZE = 1024 * 1024 * 4  # 断点续传分块大小，该参数为接口规格，暂不支持修改
 
 _config = {
     'default_zone': zone.Zone(),
@@ -18,6 +18,7 @@ _config = {
     'connection_timeout': 30,  # 链接超时为时间为30s
     'connection_retries': 3,  # 链接重试次数为3次
     'connection_pool': 10,  # 链接池个数为10
+    'default_upload_threshold': 2 * _BLOCK_SIZE  # put_file上传方式的临界默认值
 }
 
 
@@ -28,7 +29,7 @@ def get_default(key):
 def set_default(
         default_zone=None, connection_retries=None, connection_pool=None,
         connection_timeout=None, default_rs_host=None, default_uc_host=None,
-        default_rsf_host=None, default_api_host=None):
+        default_rsf_host=None, default_api_host=None, default_upload_threshold=None):
     if default_zone:
         _config['default_zone'] = default_zone
     if default_rs_host:
@@ -45,3 +46,5 @@ def set_default(
         _config['connection_pool'] = connection_pool
     if connection_timeout:
         _config['connection_timeout'] = connection_timeout
+    if default_upload_threshold:
+        _config['default_upload_threshold'] = default_upload_threshold
